@@ -6,6 +6,7 @@ Read and write SAS XPORT/XPT-format files.
 import enum
 import logging
 import re
+import string
 import struct
 import textwrap
 import warnings
@@ -16,16 +17,56 @@ from io import StringIO
 # Community Packages
 import pandas as pd
 
-from .__about__ import __version__  # noqa: F401
+from .__about__ import __version__  # noqa: F401 module imported but unused
 
 LOG = logging.getLogger(__name__)
 
 __all__ = [
     'Library',
     'Member',
+    'NaN',
 ]
 
 
+class SpecialMissingValue(float):
+    """
+    Special missing values.
+
+    SAS supports 27 special missing values, allowing the categorization of
+    missing data by tagging or labeling missing values using the letters A to Z
+    or an underscore.
+    """
+
+    pattern = re.compile(b'^(?P<tag>[A-Z_])' + b'\x00' * 7 + b'$')
+
+    def __bytes__(self):
+        """
+        XPORT-format byte string.
+        """
+        return self.name.encode('ascii') + b'\x00' * 7
+
+    @classmethod
+    def unpack(cls, bytestring):
+        """
+        Create a ``SpecialMissingValue`` from an XPORT-format bytestring.
+
+        For compatibility with standard Python tools, ``xport.v56.ibm_to_ieee``
+        creates regular float NaNs rather than special missing values.
+        """
+        match = cls.pattern.match(bytestring)
+        return NaN(match['tag'].decode('ascii'))
+
+
+NaN = enum.Enum(
+    value='NaN',
+    names={c: float('nan')
+           for c in '_' + string.ascii_uppercase},
+    module=__name__,
+    type=SpecialMissingValue,
+)
+NaN.__doc__ = SpecialMissingValue.__doc__
+
+
 class VariableType(enum.IntEnum):
     """
     SAS variables can be either Numeric or Character type.
@@ -293,6 +334,7 @@ class Variable(pd.Series):
         """
         Initialize SAS variable metadata.
         """
+        # TODO: Consider validating that the name isn't blank.
         metadata = {
             'label': label,
             'vtype': vtype,
@@ -397,7 +439,6 @@ class Dataset(pd.DataFrame):
         'modified',
         'sas_os',
         'sas_version',
-        # TODO: Consider including dataset type: {'DATA', 'VIEW', ''}.
     ]
 
     def copy_metadata(self, other):
@@ -454,6 +495,9 @@ class Dataset(pd.DataFrame):
         """
         Initialize SAS dataset metadata.
         """
+        # TODO: Consider validating dataset type: {'DATA', 'VIEW', 'CATALOG'}.
+        #       I think only 'DATA' is supported by the XPORT format.
+        # TODO: Consider validating that the name isn't blank.
         metadata = {
             'name': name,
             'dataset_label': dataset_label,
@@ -691,10 +735,12 @@ def from_rows(iterable, fp):
 
 def from_dataframe(dataframe, fp):
     """
-    Write a Pandas ``DataFrame`` to an open file-like object, ``fp``, in
-    XPT-format.
+    Write a Pandas ``DataFrame`` to an XPORT-format file.
+
+    Serialize to ``fp``, an open file-like object.
     """
     # Avoid circular import problems.
+    # Xport Modules
     from xport.v56 import dump
     warnings.warn('Please use ``xport.v56.dump`` in the future', DeprecationWarning)
     library = Library([Dataset(dataframe)])
@@ -704,6 +750,7 @@ def from_dataframe(dataframe, fp):
 def to_rows(fp):
     """
     Read a file in XPT-format and return rows.
+
     Deserialize ``fp`` (a ``.read()``-supporting file-like object
     containing an XPT document) to a list of rows. As XPT files are
     encoded in their own special format, the ``fp`` object must be in
@@ -717,6 +764,7 @@ def to_rows(fp):
 def to_columns(fp):
     """
     Read a file in XPT-format and return columns as a dict of lists.
+
     Deserialize ``fp`` (a ``.read()``-supporting file-like object
     containing an XPT document) to a list of rows. As XPT files are
     encoded in their own special format, the ``fp`` object must be in
@@ -729,6 +777,7 @@ def to_columns(fp):
 def to_numpy(fp):
     """
     Read a file in SAS XPT format and return a NumPy array.
+
     Deserialize ``fp`` (a ``.read()``-supporting file-like object
     containing an XPT document) to a list of rows. As XPT files are
     encoded in their own special format, the ``fp`` object must be in
@@ -740,12 +789,14 @@ def to_numpy(fp):
 def to_dataframe(fp):
     """
     Read a file in SAS XPT format and return a Pandas DataFrame.
+
     Deserialize ``fp`` (a ``.read()``-supporting file-like object
     containing an XPT document) to a list of rows. As XPT files are
     encoded in their own special format, the ``fp`` object must be in
     bytes-mode.
     """
     # Avoid circular import problems.
+    # Xport Modules
     from xport.v56 import load
     warnings.warn('Please use ``xport.v56.load`` in the future', DeprecationWarning)
     library = load(fp)
@@ -755,6 +806,8 @@ def to_dataframe(fp):
 
 class Reader:
     """
+    Read records from a SAS Transport (XPORT) file.
+
     Deserialize ``self._fp`` (a ``.read()``-supporting file-like object
     containing an XPT document) to a Python object.
 
